searchNodes=[{"doc":"This module can be used to establish a pool of Consumer workers. Example usage rabbit_mq allows you to design consistent, SDK-like Consumers. ℹ️ The following example assumes that the &quot;customer/customer.updated&quot; queue already exists. First, define your (ideally domain-specific) Consumer: defmodule CustomerConsumer do use RabbitMQ.Consumer, queue: &quot;customer/customer.updated&quot;, worker_count: 3 require Logger def consume(payload, meta, channel) do Logger.info(payload) ack(channel, meta.delivery_tag) end end Then, start as normal under your existing supervision tree: children = [ Topology, CustomerConsumer, CustomerProducer, # ...and more ] Supervisor.start_link(children, strategy: :one_for_one) As messages are published onto the &quot;customer/customer.updated&quot; queue, consume/3 (in CustomerConsumer) will be invoked. ⚠️ Please note that automatic message acknowledgement is disabled in rabbit_mq, therefore it&#39;s your responsibility to ensure messages are ack&#39;d or nack&#39;d. ℹ️ Please consult the Consumer Acknowledgement Modes and Data Safety Considerations for more details. Configuration The following options can be used with RabbitMQ.Consumer; :prefetch_count; limits the number of unacknowledged messages on a channel. Please consult the Consumer Prefetch section for more details. Defaults to 10. :queue; the name of the queue from which the Consumer should start consuming. For exclusive queues, please see the Exclusive Queues section further below. Defaults to &quot;&quot;. :worker_count; number of workers to be spawned. Cannot be greater than :max_channels_per_connection set in config. Defaults to 3. When you use RabbitMQ.Consumer, a few things happen; The module turns into a GenServer. The server starts and supervises the desired number of workers. consume/3 is passed as a callback to each worker when start_link/1 is called. If an exclusive queue is requested, it will be declared and bound to the Consumer. Each worker starts consuming from the queue provided, calls consume/3 for each message consumed. ack/2, nack/2, and reject/2 become automatically available. consume/3 needs to be defined with the following signature; @type payload :: String.t() @type meta :: map() @type channel :: AMQP.Channel.t() @type result :: :ok | {:error, :retry} | {:error, term()} consume(payload(), meta(), channel()) :: result() Exclusive queues If you want to consume from an exclusive queue, simply use one of the following tuples in the configuration; {exchange, routing_key} {exchange, routing_key, opts} {exchange, routing_key, queue_name} {exchange, routing_key, queue_name, opts} ℹ️ At a minimum, the exchange and the routing_key are required. defmodule CustomerConsumer do use RabbitMQ.Consumer, queue: {&quot;customer&quot;, &quot;customer.*&quot;}, worker_count: 3 # Define `consume/3` as normal. end This will ensure that the queue is declared and correctly bound before the Consumer workers start consuming messages off it.","ref":"RabbitMQ.Consumer.html","title":"RabbitMQ.Consumer","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor.","ref":"RabbitMQ.Consumer.html#child_spec/1","title":"RabbitMQ.Consumer.child_spec/1","type":"function"},{"doc":"Starts this module as a process via GenServer.start_link/3. Only used by the module&#39;s child_spec.","ref":"RabbitMQ.Consumer.html#start_link/2","title":"RabbitMQ.Consumer.start_link/2","type":"function"},{"doc":"Invoked when the server is about to exit. It should do any cleanup required. See https://hexdocs.pm/elixir/GenServer.html#c:terminate/2 for more details.","ref":"RabbitMQ.Consumer.html#terminate/2","title":"RabbitMQ.Consumer.terminate/2","type":"function"},{"doc":"The internal state held in the RabbitMQ.Consumer server. :connection; holds the dedicated AMQP.Connection. :workers; the children started under the server. The server acts as a Supervisor.","ref":"RabbitMQ.Consumer.State.html","title":"RabbitMQ.Consumer.State","type":"module"},{"doc":"The single Consumer worker used to consume messages from a queue. Calls consume/3 defined in the parent module for each message consumed.","ref":"RabbitMQ.Consumer.Worker.html","title":"RabbitMQ.Consumer.Worker","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor.","ref":"RabbitMQ.Consumer.Worker.html#child_spec/1","title":"RabbitMQ.Consumer.Worker.child_spec/1","type":"function"},{"doc":"","ref":"RabbitMQ.Consumer.Worker.html#start_link/1","title":"RabbitMQ.Consumer.Worker.start_link/1","type":"function"},{"doc":"Invoked when the server is about to exit. It should do any cleanup required. See https://hexdocs.pm/elixir/GenServer.html#c:terminate/2 for more details.","ref":"RabbitMQ.Consumer.Worker.html#terminate/2","title":"RabbitMQ.Consumer.Worker.terminate/2","type":"function"},{"doc":"The internal state held in the RabbitMQ.Consumer.Worker server. :channel; holds the dedicated AMQP.Channel. :consume_cb; the callback invoked for each message consumed. :consumer_tag; the unique consumer identifier.","ref":"RabbitMQ.Consumer.Worker.State.html","title":"RabbitMQ.Consumer.Worker.State","type":"module"},{"doc":"This module can be used to establish a pool of Producer workers. Example usage rabbit_mq allows you to design consistent, SDK-like Producers. ℹ️ The following example assumes that the &quot;customer&quot; exchange already exists. First, define your (ideally domain-specific) Producer: defmodule CustomerProducer do use RabbitMQ.Producer, exchange: &quot;customer&quot;, worker_count: 3 def customer_updated(updated_customer) when is_map(updated_customer) do # See https://hexdocs.pm/amqp/AMQP.Basic.html#publish/5 for all available options. opts = [ content_type: &quot;application/json&quot;, correlation_id: UUID.uuid4(), mandatory: true ] payload = Jason.encode!(updated_customer) publish(payload, &quot;customer.updated&quot;, opts) end end Then, start as normal under your existing supervision tree: children = [ Topology, CustomerConsumer, CustomerProducer, # ...and more ] Supervisor.start_link(children, strategy: :one_for_one) Finally, call the exposed methods from your application: CustomerProducer.customer_updated(updated_customer) Configuration The following options can be used with RabbitMQ.Producer; :confirm_type; publisher acknowledgement mode. Only :async is supported for now. Please consult the Publisher Confirms section for more details. Defaults to :async. :exchange; the name of the exchange onto which the producer workers will publish. Defaults to &quot;&quot;. :worker_count; number of workers to be spawned. Cannot be greater than :max_channels_per_connection set in config. Defaults to 3. When you use RabbitMQ.Consumer, a few things happen; The module turns into a GenServer. The server starts and supervises the desired number of workers. publish/3 becomes available in your module. publish/3 is a private function with the following signature; @type payload :: String.t() @type routing_key :: String.t() @type opts :: keyword() @type result :: :ok | {:error, :correlation_id_missing} publish(payload(), routing_key(), opts()) :: result() ⚠️ Please note that correlation_id is always required and failing to provide it will result in an exception. ℹ️ To see which options can be passed as opts to publish/3, visit https://hexdocs.pm/amqp/AMQP.Basic.html#publish/5.","ref":"RabbitMQ.Producer.html","title":"RabbitMQ.Producer","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor.","ref":"RabbitMQ.Producer.html#child_spec/1","title":"RabbitMQ.Producer.child_spec/1","type":"function"},{"doc":"Starts this module as a process via GenServer.start_link/3. Only used by the module&#39;s child_spec.","ref":"RabbitMQ.Producer.html#start_link/2","title":"RabbitMQ.Producer.start_link/2","type":"function"},{"doc":"Invoked when the server is about to exit. It should do any cleanup required. See https://hexdocs.pm/elixir/GenServer.html#c:terminate/2 for more details.","ref":"RabbitMQ.Producer.html#terminate/2","title":"RabbitMQ.Producer.terminate/2","type":"function"},{"doc":"The internal state held in the RabbitMQ.Producer server. :connection; holds the dedicated AMQP.Connection. :workers; the children started under the server. The server acts as a Supervisor. Enables round-robin command dispatch. :worker_count; a simple worker count tracker. Used in round-robin command dispatch logic. :worker_offset; a simple tracker to determine which worker to call next. Used in round-robin command dispatch logic.","ref":"RabbitMQ.Producer.State.html","title":"RabbitMQ.Producer.State","type":"module"},{"doc":"The single Producer worker used to publish messages onto an exchange.","ref":"RabbitMQ.Producer.Worker.html","title":"RabbitMQ.Producer.Worker","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor.","ref":"RabbitMQ.Producer.Worker.html#child_spec/1","title":"RabbitMQ.Producer.Worker.child_spec/1","type":"function"},{"doc":"Starts this module as a process via GenServer.start_link/2. Only used by the parent module which acts as a Supervisor.","ref":"RabbitMQ.Producer.Worker.html#start_link/1","title":"RabbitMQ.Producer.Worker.start_link/1","type":"function"},{"doc":"Invoked when the server is about to exit. It should do any cleanup required. See https://hexdocs.pm/elixir/GenServer.html#c:terminate/2 for more details.","ref":"RabbitMQ.Producer.Worker.html#terminate/2","title":"RabbitMQ.Producer.Worker.terminate/2","type":"function"},{"doc":"The internal state held in the RabbitMQ.Producer.Worker server. :channel; holds the dedicated AMQP.Channel. :outstanding_confirms; holds the reference to the protected ets table used to track outstanding Publisher ack or nack confirms.","ref":"RabbitMQ.Producer.Worker.State.html","title":"RabbitMQ.Producer.Worker.State","type":"module"},{"doc":"A convenience module that can be used to establish the (RabbitMQ) network topology. First, create a module that uses RabbitMQ.Topology to define exchanges and their corresponding bindings as shown below. ⚠️ Please note that exclusive queues cannot be configured here. You may need to consult the RabbitMQ.Consumer module for details on how exclusive queues can be set up and used. defmodule Topology do use RabbitMQ.Topology, exchanges: [ {&quot;customer&quot;, :topic, [ {&quot;customer.created&quot;, &quot;customer/customer.created&quot;, durable: true}, {&quot;customer.updated&quot;, &quot;customer/customer.updated&quot;, durable: true}, {&quot;#&quot;, &quot;customer/#&quot;} ], durable: true} ] end Then, simply add this module to your supervision tree, before any Consumers or Producers that rely on the exchanges configured within it start. ⚠️ Please note that the Topology module will terminate gracefully as soon as the network is configured. children = [ Topology, MyConsumer, MyProducer, # ...and more ] Supervisor.start_link(children, strategy: :one_for_one)","ref":"RabbitMQ.Topology.html","title":"RabbitMQ.Topology","type":"module"},{"doc":"The macro to use this module. Available options: exchanges: [ { # Exchange name &quot;customer&quot;, # Exchange type, only topic is supported at the moment :topic, # List of bindings [ { # Routing/binding key &quot;#&quot;, # Queue name &quot;customer/#&quot;, # Queue opts (optional) durable: true } ], # Exchange opts durable: true } ]","ref":"RabbitMQ.Topology.html#__using__/1","title":"RabbitMQ.Topology.__using__/1","type":"macro"},{"doc":"Overview rabbit_mq is an opinionated RabbitMQ client to help you build balanced and consistent Consumers and Producers. The following modules are provided; RabbitMQ.Topology RabbitMQ.Consumer RabbitMQ.Producer","ref":"overview.html","title":"Overview","type":"extras"},{"doc":"The RabbitMQ modules are pre-configured with sensible defaults and follow design principles that improve and delicately balance both performance and reliability. This has been possible through a) extensive experience of working with Elixir and RabbitMQ in production; and b) meticulous consultation of the below (and more) documents and guides. ⚠️ While most of the heavy-lifting is provided by the library itself, reading through the documents below before running any application in production is thoroughly recommended. Connections Channels Reliability Guide Publisher Confirms Consumer Acknowledgements and Publisher Confirms Consumer Acknowledgement Modes and Data Safety Considerations Consumer Prefetch Production Checklist RabbitMQ Best Practices RabbitMQ Best Practice for High Performance (High Throughput) Performance ℹ️ This section is currently being completed. Thank you for your understanding and patience. Talk about async mode for publisher confirms Talk about connection and channel pooling Talk about workers Reliability ℹ️ This section is currently being completed. Thank you for your understanding and patience. Talk about connection and channel pooling Talk about workers","ref":"overview.html#balanced-performance-and-reliability","title":"Overview - Balanced performance and reliability","type":"extras"},{"doc":"ℹ️ This section is currently being completed. Thank you for your understanding and patience. The RabbitMQ modules are designed to help you build consistent, SDK-like Consumers and Producers. defmodule CustomerProducer do use RabbitMQ.Producer, exchange: &quot;customer&quot; @doc &quot;&quot;&quot; Publishes an event routed via &quot;customer.created&quot;. &quot;&quot;&quot; def customer_created(customer_id) do opts = [ content_type: &quot;application/json&quot;, correlation_id: UUID.uuid4(), mandatory: true ] payload = Jason.encode!(%{version: &quot;1.0.0&quot;, customer_id: customer_id}) publish(payload, &quot;customer.created&quot;, opts) end @doc &quot;&quot;&quot; Publishes an event routed via &quot;customer.created&quot;. &quot;&quot;&quot; def customer_updated(customer_data) do opts = [ content_type: &quot;application/json&quot;, correlation_id: UUID.uuid4(), mandatory: true ] payload = Jason.encode!(%{version: &quot;1.0.0&quot;, customer_data: customer_data}) publish(payload, &quot;customer.created&quot;, opts) end end","ref":"overview.html#consistency","title":"Overview - Consistency","type":"extras"},{"doc":"ℹ️ This section is currently being completed. Thank you for your understanding and patience. Talk about connection and channel pooling and management, supervision, workers This is a near-0 dependency library, built solely on top of amqp.","ref":"overview.html#application-design","title":"Overview - Application design","type":"extras"},{"doc":"Please consult the Configuration guide for details.","ref":"overview.html#configuration","title":"Overview - Configuration","type":"extras"},{"doc":"Configuration The following options must be configured. amqp_url; required, the broker URL. heartbeat_interval_sec; required, defines after what period of time the peer TCP connection should be considered unreachable. reconnect_interval_ms; required, the interval before another attempt to re-connect to the broker should occur. max_channels_per_connection; required, maximum number of channels per connection. Also determines the maximum number of workers per Producer/Consumer module. config :rabbit_mq, amqp_url: &quot;amqp://guest:guest@localhost:5672&quot;, heartbeat_interval_sec: 60, reconnect_interval_ms: 2500, max_channels_per_connection: 16 ⚠️ Please consult the Channels Resource Usage guide to understand how to best configure :max_channels_per_connection. ⚠️ Please consult the Detecting Dead TCP Connections with Heartbeats and TCP Keepalives guide to understand how to best configure :heartbeat_interval_sec.","ref":"configuration.html","title":"Configuration","type":"extras"},{"doc":"Topology ℹ️ This section is currently being completed. Thank you for your understanding and patience. Please refer to the RabbitMQ.Topology module for documentation.","ref":"topology.html","title":"Topology","type":"extras"},{"doc":"Consumers ℹ️ This section is currently being completed. Thank you for your understanding and patience. Please refer to the RabbitMQ.Consumer module for documentation.","ref":"consumers.html","title":"Consumers","type":"extras"},{"doc":"Producers ℹ️ This section is currently being completed. Thank you for your understanding and patience. Please refer to the RabbitMQ.Producer module for documentation.","ref":"producers.html","title":"Producers","type":"extras"}]